/***********************************************************
 * DataAccess Class
 * This is a simple data access layer intended to seperate
 * data calls from component views by putting all data access
 * in a reusable class.
 * 
 * Brandon Ellis - 2007
 * brandonthedeveloper@gmail.com
 * http://www.brandonellis.org	
 * 
 * Usage
 * Import the class to your mxml file.
 * 		import classes.DataAccess;
 *
 * Create an 'bindable' instance of the DataAccess Class in the mxml file;
 *		[Bindable]
		private var da:DataAccess;
 * 
 * Pass in the file path to the database
 *		da = new DataAccess("app-resource:/friends.db");
 * 
 * Open a connection to the database and pass in the initial sql statement to run
 * 		da.openConnection("select * from friends") - this is optional;
			
 * Set the receiving data component's dataProvider to be the DataAccess.dbResult property.
 * dataProvider="{da.dbResult}"
 * 
 * For more information on creating a local instance of the DataAccess Class take a look at the 'main.mxml' file
***********************************************************/
package infrastructure.database
{
// imported classes
import flash.data.SQLConnection;
import flash.data.SQLResult;
import flash.data.SQLStatement;
import flash.errors.SQLError;
import flash.events.SQLErrorEvent;
import flash.events.SQLEvent;
import flash.filesystem.File;

import framework.dialogs.MessageBox;

import infrastructure.statemanagement.StateMgr;

import mx.collections.ArrayCollection;
import mx.controls.Alert;

public class DataAccess
{
	// private variables for the class
	private var _dbFilePath:String;
	
	private var _sql2:String="";
	private var db:File;
	private var sqlConnection:SQLConnection;
	private var handlers:Array = new Array();
	//private var sqlStatement:SQLStatement;	

	// this ArrayCollection is used as the data provider 
	// dbResult getter
	[Bindable]
	public var ResultSet:ArrayCollection = new ArrayCollection();
	
	// constructor
	public function DataAccess(dbFilePath:String) {
		_dbFilePath = dbFilePath;
		//openConnection();
	}
	
/***************** Open Connection *************************
* Create a sqlConnection and open the database
***********************************************************/
	public function openConnection(sql:String=""):void {
		
		_sql2 = sql;
		// the database
		db = new File(_dbFilePath);
	    // instance of the SQLConnection
	    sqlConnection = new SQLConnection();
	    sqlConnection.addEventListener(SQLEvent.OPEN, sqlConnectionOpenHandler);
	    sqlConnection.addEventListener(SQLErrorEvent.ERROR, sqlConnectionErrorHandler);
	    // open the database
	    if(db.exists)
	    {
	    sqlConnection.open(db);
	    }
	    else
	    {
	      MessageBox.Show("Database files are missing;");
	    }		
	}
	
	public function closeConnection():void
	{
		if (sqlConnection != null)
			sqlConnection.close(); 
	}
	
	public function isConnected():Boolean
	{
		if (sqlConnection != null)
			return sqlConnection.connected;
		return false;
	}
	
	// when the connection is opened
	private function sqlConnectionOpenHandler(evt:SQLEvent):String {
	    if (evt.type == "open" && _sql2 != "") {
	        // the initial database view to show
	       _sql2 = _sql2.replace("$USERID$",StateMgr.GetInstance().UserId);
		 return executeSQL(_sql2, selectResultHandler);
	    }
	    return "";
	}
	
	// when the connection has an error
	private function sqlConnectionErrorHandler(evt:SQLErrorEvent):void {
		Alert.show("You have problems... " +  evt.error.message);
	}	
	
/***************** Select **********************************
 * the method 'executeSQL' is used by the Select, Insert, Update and Delete methods
 * as a generic/reusable function to connect to the database and execute the passed in sql query.
 * the handler to fire 'onResult' is passed in as the second argument
***********************************************************/
	public function DataAccessSelect(sql:String,handler:Function=null):String {
		sql = sql.replace("$USERID$",StateMgr.GetInstance().UserId);
		return executeSQL(sql, selectResultHandler,handler);
	}
	
	
	
	
	
	private function selectResultHandler(evt:SQLEvent):void {
		var result:SQLResult = evt.currentTarget.getResult();	
			
		// reset the _sql2 variable to an empty string
		if (_sql2 != "") _sql2 = "";	
		
		if(handlers[evt.currentTarget] != null)
		{
			var resultSet:ArrayCollection = null;
			if (result != null)
			 {	
				resultSet = new ArrayCollection(result.data);
			 }
			 else
			 {
			 	resultSet = new ArrayCollection();
			 }
			 
			 if(handlers[evt.currentTarget] != null)
			 {
				handlers[evt.currentTarget](resultSet);
			 }
		}	
		else
		{
			if (result != null)
			 {	
			 	ResultSet.removeAll();
			 	if(result.data != null)
			 	{
				 	for (var i:int =0; i<result.data.length; i++)
				 	{
						ResultSet.addItem(result.data[i]);
				 	}
			 	}
			 }	
		}
		
	}
	
	
/***************** Execute **********************************
***********************************************************/
	public function DataAccessExecute(sql:String, sql2:String=""):String {
		// give the variable _sql2 its value
	    this._sql2 = sql2;
		return executeSQL(sql, executeResultHandler);
	}
	
	private function executeResultHandler(evt:SQLEvent):void {
		// if a change has been made to the database and _sql2 is not an empty string
	    if (sqlConnection.totalChanges >= 1 && _sql2 != "") {
	        DataAccessSelect(_sql2);
	    }
	}
	
	
/***************** Insert **********************************
***********************************************************/
	public function DataAccessInsert(sql:String, sql2:String=""):String {
		// give the variable _sql2 its value
	    this._sql2 = sql2;
		return executeSQL(sql, insertResultHandler);
	}
	
	private function insertResultHandler(evt:SQLEvent):void {
		// if a change has been made to the database and _sql2 is not an empty string
	    if (sqlConnection.totalChanges >= 1 && _sql2 != "") {
	        DataAccessSelect(_sql2);
	    }
	}
	
/***************** Update **********************************
***********************************************************/
	public function DataAccessUpdate(sql:String, sql2:String=""):String {
		this._sql2 = sql2;
		return executeSQL(sql, updateResultHandler);
	}
	
	private function updateResultHandler(evt:SQLEvent):void {
	    if (sqlConnection.totalChanges >= 1 && _sql2 != "") {
	        DataAccessSelect(_sql2);
	    }
	}	
	
/***************** Delete **********************************	
***********************************************************/
	public function DataAccessDelete(sql:String, sql2:String=""):String {
		this._sql2 = sql2;
		return executeSQL(sql, deleteResultHandler);
	}
	
	private function deleteResultHandler(evt:SQLEvent):void {
	    if (sqlConnection.totalChanges >= 1 && _sql2 != "") {
	        DataAccessSelect(_sql2);
	    }
	}
			
/***************** Execute **********************************
 * the executeSQL function returns as  String so you could (optionally) read back any 
 * SQLError messages.
***********************************************************/	
	private function executeSQL(sql:String, handler:Function,customhandler:Function=null):String {
		var statusMsg:String = "";
		try {
			// eventlistener function to fire off on result
			var _handler:Function = handler;
			var sqlStmt:SQLStatement = new SQLStatement();
		    sqlStmt.sqlConnection = sqlConnection;	
		    // sql to run
		    sqlStmt.text = sql;
		    
		    sqlStmt.addEventListener(SQLEvent.RESULT, _handler);
		  	handlers[sqlStmt] = customhandler;
		    // execute the sqlStatement
		    sqlStmt.execute();	
		} catch (e:SQLError) {
			// catch any errors
			statusMsg = e.message;	
			MessageBox.Show(statusMsg);		
		}	
		
		return statusMsg;	
	}
} // class
} // namespace